#!/usr/bin/env python
# coding: utf-8

# In[9]:


import pandas as pd
import numpy as np
from datetime import timedelta
import glob
import os


# In[14]:


# Define the folder containing the Excel files
files_folder = os.path.join(os.getcwd(), 'files_folder')  
file_pattern = os.path.join(files_folder, '*.xlsx')  # Matches all .xlsx files in the folder

# Get a list of all .xlsx files in the folder
files = glob.glob(file_pattern)

# Ensure at least one file exists
if not files:
    raise FileNotFoundError("No .xlsx files found in the directory.")

# Read the first available file
selected_file = files.pop(0)  # Remove the first file from the list
print(f"Selected file: {selected_file}")

# Read the first file into a DataFrame
df = pd.read_excel(selected_file)
df['Time'] = pd.to_datetime(df['Date']).dt.time
df['Date'] = pd.to_datetime(df['Date']).dt.date
df.sort_values(by='Date', axis=0, inplace=True)

# Extract laptop ID and convert to integer if available
laptop_id = df[df['Field Name'] == 'Barcode']['New Value']
df['Id'] = int(laptop_id.iloc[0]) if not laptop_id.empty else None  # FIXED: Convert to int safely

# Function to read additional files
def read_file_and_do_some_operation(path_to_file):
    print(f"Processing: {path_to_file}")
    df_return = pd.read_excel(path_to_file)
    df_return['Time'] = pd.to_datetime(df_return['Date']).dt.time
    df_return['Date'] = pd.to_datetime(df_return['Date']).dt.date
    df_return.sort_values(by='Date', axis=0, inplace=True)

    laptop_id = df_return[df_return['Field Name'] == 'Barcode']['New Value']
    df_return['Id'] = int(laptop_id.iloc[0]) if not laptop_id.empty else None  # FIXED: Convert to int safely
    return df_return

# Process remaining files and merge into a single DataFrame
df_concat = df.copy()
for file in files:
    df_temp = read_file_and_do_some_operation(file)
    df_concat = pd.concat([df_concat, df_temp], ignore_index=True)

# Final DataFrame after merging all files
print("Final DataFrame:")
print(df_concat.head())


# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:





# In[15]:


len(df_concat['Id'].unique())


# In[16]:


df_final = pd.DataFrame(columns=['Laptop_Id', 'Dates Loaned', 'Dates Returned','Time of Loan','Time of Return'])


# In[17]:


for index, row in df_concat[['Date', 'Field Name', 'Old Value', 'New Value','Time', 'Id']].iterrows():
    date = row['Date']
    field_name = row['Field Name']
    old_value = row['Old Value']
    new_value = row['New Value']
    Id = row['Id']
    time=row['Time']
    if(field_name=='Process type'):
        if(pd.isna(old_value) and new_value=='Loan'):
            temp_date=date
            temp_time=time
            temp_id=Id
        elif(pd.isna(new_value) and old_value=='Loan' and temp_id==Id):
            # Add a new row to the original DataFrame
            df_final.loc[len(df_final)] = [Id,temp_date,date,temp_time,time]
            temp_id=''
            


# In[18]:


df_final[df_final['Dates Loaned']==pd.to_datetime('2019-11-24').date()]


# In[19]:


def average_laptops_checked_out(date_start,date_end):
    date_start=pd.to_datetime(date_start).date()
    date_end=pd.to_datetime(date_end).date()
    current_date = date_start
    count=0
    total_days=0
    # Iterate through the days from start to end
    while current_date <= date_end:
        count=count+len(df_final[((df_final['Dates Loaned']<=current_date) & (df_final['Dates Returned']>current_date)) | ((df_final['Dates Loaned']==current_date) & (df_final['Dates Returned']==current_date))]['Laptop_Id'].unique())
        total_days=total_days+1
        # Increment the date by one day
        current_date += timedelta(days=1)
        #print(df_final[((df_final['Dates Loaned']<=current_date) & (df_final['Dates Returned']>current_date)) | ((df_final['Dates Loaned']==current_date) & (df_final['Dates Returned']==current_date))])
    print('Average laptop checked out per day during the given time frame = '+ str(count/total_days))
    return str(count/total_days)
    '''unique_laptop_id_loaned=df_final[(df_final['Dates Loaned']>=date_start) & (df_final['Dates Returned']<=date_end)]['Laptop_Id'].unique()
    unique_laptop_id_present=df_final['Laptop_Id'].unique()
    print('For the time dates '+ str(date_start)+ ' and ' + str(date_end))
    print('Total no of laptops present with us = '+str(len(unique_laptop_id_present)))
    print('Total no of laptop which were checked out atleast once during the above time = '+str(len(unique_laptop_id_loaned)))'''


# In[20]:


df_final['Laptop_Id']=df_final['Laptop_Id'].astype(str)
df_final.to_excel('assets_loan_date.xlsx', index=False)


# In[21]:


df_for_line_graph=df_final.copy()


# In[22]:


#average_days=average_laptops_checked_out('2019-11-04','2021-11-07')


# In[ ]:





# In[23]:


import pandas as pd
import matplotlib.pyplot as plt

def average_asset_checkout(date_start, date_end):
    # Convert start and end dates to datetime (not date)
    date_start = pd.to_datetime(date_start)
    date_end = pd.to_datetime(date_end)

    # Ensure that 'Dates Loaned' is of datetime type (in case it's not)
    df_for_line_graph['Dates Loaned'] = pd.to_datetime(df_for_line_graph['Dates Loaned'])
    # Filter the dataset within the specified date range
    df_temp = df_for_line_graph[(df_for_line_graph['Dates Loaned'] >= date_start) & (df_for_line_graph['Dates Loaned'] <= date_end)]
    
    # Extract Month-Year
    df_temp['Month-Year'] = df_temp['Dates Loaned'].dt.to_period('M')

    return df_temp

import matplotlib.dates as mdates

def plot_monthly_loan_trend(date_start, date_end):
    # Get the filtered dataset with Month-Year
    df_temp = average_asset_checkout(date_start, date_end)
    
    # Count the number of loans for each Month-Year
    monthly_loans = df_temp.groupby('Month-Year').size()
    # Convert the PeriodIndex to a datetime format for plotting
    monthly_loans.index = monthly_loans.index.to_timestamp()
    
    # Plot the data
    plt.figure(figsize=(12, 6))
    plt.plot(monthly_loans.index, monthly_loans.values, marker='o', label='Laptops Loaned')
    
    # Format the x-axis to display Month and Year (e.g., Jan 2023)
    plt.gca().xaxis.set_major_formatter(mdates.DateFormatter('%b %Y'))  # Month and Year format
    plt.gca().xaxis.set_major_locator(mdates.MonthLocator())  # Tick for each month

    # Adjust tick parameters for better visualization
    plt.xticks(rotation=45, ha='right')
    plt.title('Monthly Laptop Loan Trend')
    plt.xlabel('Month-Year')
    plt.ylabel('Number of Loans')
    plt.grid(True)
    plt.legend()
    plt.tight_layout()  # Adjust layout to prevent overlap
    plot_path = os.path.join("static", "loan_trend.png")
    plt.savefig(plot_path)
    plt.close()

    # Return the plot path and monthly loans data as a dictionary
    return "\static\loan_trend.png", monthly_loans.reset_index(name='Number of Loans')


# In[24]:


'''start_date='2023-01-01'
end_date='2023-07-31'
plot_monthly_loan_trend(start_date, end_date)'''


# In[25]:


df_for_line_graph_hours=df_final.copy()


# In[26]:


df_for_line_graph_hours['loaned_day_of_week'] = pd.to_datetime(df_for_line_graph_hours['Dates Loaned']).dt.day_name()


# In[27]:


#df_for_line_graph_hours


# In[28]:


import pandas as pd
import matplotlib.pyplot as plt

def plot_loan_times(start_date: str, end_date: str):
    # Convert start_date and end_date to pandas Timestamp objects
    start_date = pd.to_datetime(start_date)
    end_date = pd.to_datetime(end_date)
    
    # Ensure 'Dates Loaned' is a datetime object and remove the time part for comparison
    df_for_line_graph_hours['Dates Loaned'] = pd.to_datetime(df_for_line_graph_hours['Dates Loaned']).dt.date
    
    # Filter data within the date range by comparing with datetime.date
    filtered_df = df_for_line_graph_hours[
        (df_for_line_graph_hours['Dates Loaned'] >= start_date.date()) & 
        (df_for_line_graph_hours['Dates Loaned'] <= end_date.date())
    ]
    
    # Convert 'Time of Loan' to datetime and extract the hour (from 24-hour format)
    filtered_df['Time of Loan'] = pd.to_datetime(filtered_df['Time of Loan'], format='%H:%M:%S').dt.time
    # Combine 'Time of Loan' with a dummy date to get full datetime for extracting hour
    filtered_df['Loan Time'] = pd.to_datetime('1900-01-01 ' + filtered_df['Time of Loan'].astype(str))
    
    # Extract hour from 'Loan Time' and add it as a new column
    filtered_df['Loan Hour'] = filtered_df['Loan Time'].dt.hour
    
    # Group by 'Dates Loaned' and 'Loan Hour' to get the count of loans per hour per day
    loan_counts = filtered_df.groupby([filtered_df['Dates Loaned'], 'Loan Hour']).size().unstack(fill_value=0)

    # Select the relevant hours of the day (e.g., from 6 AM to 10 PM)
    loan_counts = loan_counts.loc[:, 6:22]  # 6 AM to 10 PM

    
    # Ensure all days have entries for all hours (from 6 to 22). Add missing hours with 0 loans
    all_hours = range(6, 23)  # Hours from 6 AM to 10 PM
    loan_counts = loan_counts.reindex(columns=all_hours, fill_value=0)
    print (loan_counts)
    # Transpose the DataFrame to swap rows and columns
    loan_counts_T = loan_counts.T

    # Plotting
    ax = loan_counts_T.plot(kind='line', figsize=(20, 6), marker='o', linestyle='-', linewidth=2)

    ax.set_title(f"Hourly Loan Counts from {start_date.date()} to {end_date.date()}")
    ax.set_xlabel('Hour of Day')
    ax.set_ylabel('Number of Loans')
    ax.set_xticks(range(6, 23))  # Hour ticks from 6 AM to 10 PM
    ax.set_xticklabels([f"{i}:00" for i in range(6, 23)])  # Label the hours
    ax.legend(title='Day', bbox_to_anchor=(1.05, 1), loc='upper left')  # Each line represents a day
    plt.tight_layout()
    plt.show()


# In[ ]:





# In[29]:


'''import pandas as pd
import matplotlib.pyplot as plt

def plot_average_loans_by_day_of_week(start_date: str, end_date: str):
    # Convert start_date and end_date to pandas Timestamp objects
    start_date = pd.to_datetime(start_date)
    end_date = pd.to_datetime(end_date)
    
    # Ensure 'Dates Loaned' is a datetime object
    df_for_line_graph_hours['Dates Loaned'] = pd.to_datetime(df_for_line_graph_hours['Dates Loaned'])
    
    # Filter data within the date range
    filtered_df = df_for_line_graph_hours[
        (df_for_line_graph_hours['Dates Loaned'] >= start_date) & 
        (df_for_line_graph_hours['Dates Loaned'] <= end_date)
    ]
    
    # Extract hour from 'Time of Loan'
    filtered_df['Loan Hour'] = pd.to_datetime(filtered_df['Time of Loan'], format='%H:%M:%S').dt.hour
    
    # Group by 'loaned_day_of_week' and 'Loan Hour', then calculate the average
    loan_counts = (
        filtered_df
        .groupby(['loaned_day_of_week', 'Loan Hour'])
        .size()
        .unstack(fill_value=0)
        .groupby(level=0)
        .mean()  # Calculate the average for each day of the week and hour
    )
    
    # Select hours between 6 PM and 10 PM
    loan_counts = loan_counts.loc[:, 6:22]  # 6 AM to 10 PM

    # Ensure all days of the week are represented, even if no data exists
    all_days = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday']
    loan_counts = loan_counts.reindex(all_days, fill_value=0)
    
    # Plotting
    ax = loan_counts.T.plot(kind='line', figsize=(25, 8), marker='o', linestyle='-', linewidth=2)
    
    ax.set_title(f"Average Hourly Loan Counts by Day of Week ({start_date.date()} to {end_date.date()})")
    ax.set_xlabel('Hour of Day')
    ax.set_ylabel('Average Number of Loans')
    ax.set_xticks(range(6, 23))  # Hours between 6 AM and 10 PM
    ax.set_xticklabels([f"{i}:00" for i in range(6, 23)])  # Format hour labels
    ax.legend(title='Day of Week', bbox_to_anchor=(1.05, 1), loc='upper left')  # Legend for days
    plt.tight_layout()
    plot_path = os.path.join("static", "checkout_trend.png")
    plt.savefig(plot_path)
    plt.show()
    plt.close()
    #return "/static/checkout_trend.png"

# Example usage:
plot_average_loans_by_day_of_week('2019-11-12', '2019-11-13')'''


# In[30]:


import pandas as pd
import matplotlib.pyplot as plt
import os

def plot_average_loans_by_day_of_week(start_date: str, end_date: str):
    # Convert start_date and end_date to pandas Timestamp objects
    start_date = pd.to_datetime(start_date)
    end_date = pd.to_datetime(end_date)
    
    # Ensure 'Dates Loaned' is a datetime object
    df_for_line_graph_hours['Dates Loaned'] = pd.to_datetime(df_for_line_graph_hours['Dates Loaned'])
    
    # Filter data within the date range
    filtered_df = df_for_line_graph_hours[
        (df_for_line_graph_hours['Dates Loaned'] >= start_date) & 
        (df_for_line_graph_hours['Dates Loaned'] <= end_date)
    ]
    
    # Extract hour from 'Time of Loan'
    filtered_df['Loan Hour'] = pd.to_datetime(filtered_df['Time of Loan'], format='%H:%M:%S').dt.hour
    
    # Group by 'loaned_day_of_week' and 'Loan Hour', then calculate the average
    loan_counts = (
        filtered_df
        .groupby(['loaned_day_of_week', 'Loan Hour'])
        .size()
        .unstack(fill_value=0)  # Fill missing hours with 0
    )
    
    # Add missing hours (6 AM to 10 PM) for each day of the week
    all_hours = range(6, 23)  # 6 AM to 10 PM
    loan_counts = loan_counts.reindex(columns=all_hours, fill_value=0)
    
    # Ensure all days of the week are represented and start from Monday
    all_days = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday']
    loan_counts = loan_counts.reindex(all_days, fill_value=0)
    
    # Calculate the average for each day of the week and hour
    loan_counts = loan_counts.groupby(level=0).mean()
    
    # Plotting
    ax = loan_counts.T.plot(kind='line', figsize=(25, 8), marker='o', linestyle='-', linewidth=2)
    
    # Increase font size for title, axes labels, and legend
    ax.set_title(f"Average Hourly Loan Counts by Day of Week ({start_date.date()} to {end_date.date()})", fontsize=20)
    ax.set_xlabel('Hour of Day', fontsize=20)
    ax.set_ylabel('Average Number of Loans', fontsize=20)
    ax.set_xticks(range(6, 23))  # Hours between 6 AM and 10 PM
    ax.set_xticklabels([f"{i}:00" for i in range(6, 23)], fontsize=16)  # Format hour labels
    ax.tick_params(axis='y', labelsize=14)  # Increase y-axis tick label size
    
    # Explicitly order the legend and ensure labels follow 'all_days' order
    handles, labels = ax.get_legend_handles_labels()
    ordered_labels = [label for label in all_days if label in labels]
    ordered_handles = [handles[labels.index(label)] for label in ordered_labels]
    ax.legend(ordered_handles, ordered_labels, title='Day of Week', title_fontsize=16, fontsize=14, bbox_to_anchor=(1.05, 1), loc='upper left')
    
    plt.tight_layout()
    plot_path = os.path.join("static", "checkout_trend.png")
    plt.savefig(plot_path)
    plt.close()
    return "/static/checkout_trend.png"


# In[ ]:





# In[ ]:




